"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: weixinCrawler.py
@Date: Create in 2021/1/25 16:19
@Description: 对微信数据进行抓取
@ Software: PyCharm
===================================
"""

import re
import time
from time import sleep
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.chrome.options import Options


class CrawerWeixinData:

    # 初始化
    def __init__(self, query_word=None, turn_page=None):
        self.query_word = query_word
        self.turn_page = turn_page

    # 获取微博综合查询结果数据
    # https://weixin.sogou.com/weixin?type=2&query=森林火灾&page=2
    def get_Sogou_Weixin_Query_Result(self):
        chrome_options = Options()  # 实例化Option对象
        chrome_options.add_argument('--headless')  # 把Chrome浏览器设置为静默模式
        chrome_options.add_argument('--disable-gpu')  # 禁止加载图片
        dr = webdriver.Chrome(options=chrome_options)  # 设置引擎为Chrome，在后台默默运行
        print("----------------------------------------------------------")
        is_not_last_page = True
        curent_page_num = 1
        crawler_data_list = []
        while is_not_last_page and curent_page_num <= self.turn_page:
            try:
                if curent_page_num == 1:
                    url = "https://weixin.sogou.com/weixin?type=2&query=" + self.query_word
                    print("目前是第1页------")
                    dr.get(url)
                else:
                    next_page_element = self.get_next_page_togger(dr)
                    dr.execute_script("arguments[0].click();", next_page_element)
                    sleep(1)
                    print("自动获取到第{}页-----".format(str(curent_page_num)))
                # print(dr.page_source)
                curent_page_num += 1
                print("获取微信搜索数据.........")
                htmlinfo = bytes(dr.page_source, encoding='utf-8')
                soup = BeautifulSoup(htmlinfo, 'html.parser')
                news_box = soup.find_all('div', attrs={'class': 'news-box'})
                if len(news_box) > 0:
                    li_item_list = news_box[0].find_all('ul', attrs={'class': 'news-list'})[0].find_all(name='li')
                    for li_item in li_item_list:
                        # 标题,提取简介里的 【】，若没有则提取前10个字符
                        try:
                            info_title = li_item.find_all('div', attrs={
                                'class': 'txt-box'})[0].text.replace("\n", "").strip()
                        except:
                            info_title = ""
                            pass
                        # 来源
                        try:
                            info_source = \
                                li_item.find_all('div', attrs={
                                    'class': 's-p'})[0].text.replace("\n", "").strip()
                        except:
                            info_source = ""
                            pass
                        # 时间
                        try:
                            info_time = \
                                li_item.find_all('span', attrs={
                                    'class': 's2'})[0].contents
                            info_time = re.findall("'(.*?)'", info_time)[0]
                            tupTime = time.localtime(int(info_time))
                            info_time = time.strftime("%Y-%m-%d %H:%M:%S", tupTime)
                        except:
                            info_time = ""
                            pass
                        # 简介
                        try:
                            info_breif = li_item.find_all('p', attrs={'class': 'txt-info'})[0].text.replace("\n", "").strip()
                        except:
                            info_breif = ""
                            pass
                        # 链接
                        try:
                            info_href = li_item.find_all('div', attrs={'class': 'txt-box'})[0].find_all(name='a')[0].attrs.get("href")
                            info_href="https://weixin.sogou.com"+info_href
                            # TODO:需要获取永久链接
                        except:
                            info_href = ""
                            pass
                        each_data_dict = {}
                        each_data_dict["标题"] = info_title
                        each_data_dict["来源"] = info_source
                        each_data_dict["时间"] = info_time
                        each_data_dict["简介"] = info_breif
                        each_data_dict["链接"] = info_href
                        crawler_data_list.append(each_data_dict)
            except Exception as exp:
                # 通过没有下一页的提示错误来分析，是否到了最后一页
                if "Cannot read property 'click' of null" in str(exp):
                    print("当前已到最后一页")
                else:
                    print("翻页错误：{}".format(exp))
                is_not_last_page = False
                pass
        dr.quit()
        return crawler_data_list
  # 获取微博综合查询结果数据
    # https://weixin.sogou.com/weixin?type=2&query=森林火灾&page=2
    def get_Sogou_Weixin_Query_Result_Json(self,now_time,taskId):
        chrome_options = Options()  # 实例化Option对象
        chrome_options.add_argument('--headless')  # 把Chrome浏览器设置为静默模式
        chrome_options.add_argument('--disable-gpu')  # 禁止加载图片
        dr = webdriver.Chrome(options=chrome_options)  # 设置引擎为Chrome，在后台默默运行
        print("----------------------------------------------------------")
        is_not_last_page = True
        curent_page_num = 1
        crawler_data_list = []
        crawler_data_list_json = []
        data = {}
        while is_not_last_page and curent_page_num <= self.turn_page:
            try:
                if curent_page_num == 1:
                    url = "https://weixin.sogou.com/weixin?type=2&query=" + self.query_word
                    print("目前是第1页------")
                    dr.get(url)
                else:
                    next_page_element = self.get_next_page_togger(dr)
                    dr.execute_script("arguments[0].click();", next_page_element)
                    sleep(1)
                    print("自动获取到第{}页-----".format(str(curent_page_num)))
                # print(dr.page_source)
                curent_page_num += 1
                print("获取微信搜索数据.........")
                htmlinfo = bytes(dr.page_source, encoding='utf-8')
                soup = BeautifulSoup(htmlinfo, 'html.parser')
                news_box = soup.find_all('div', attrs={'class': 'news-box'})
                if len(news_box) > 0:
                    li_item_list = news_box[0].find_all('ul', attrs={'class': 'news-list'})[0].find_all(name='li')
                    for li_item in li_item_list:
                        # 标题,提取简介里的 【】，若没有则提取前10个字符
                        try:
                            info_title = li_item.find_all('div', attrs={
                                'class': 'txt-box'})[0].text.replace("\n", "").strip()
                        except:
                            info_title = ""
                            pass
                        # 来源
                        try:
                            info_source = \
                                li_item.find_all('div', attrs={
                                    'class': 's-p'})[0].text.replace("\n", "").strip()
                        except:
                            info_source = ""
                            pass
                        # 时间
                        try:
                            info_time = \
                                li_item.find_all('span', attrs={
                                    'class': 's2'})[0].contents
                            info_time = re.findall("'(.*?)'", info_time)[0]
                            tupTime = time.localtime(int(info_time))
                            info_time = time.strftime("%Y-%m-%d %H:%M:%S", tupTime)
                        except:
                            info_time = ""
                            pass
                        # 简介
                        try:
                            info_breif = li_item.find_all('p', attrs={'class': 'txt-info'})[0].text.replace("\n", "").strip()
                        except:
                            info_breif = ""
                            pass
                        # 链接
                        try:
                            info_href = li_item.find_all('div', attrs={'class': 'txt-box'})[0].find_all(name='a')[0].attrs.get("href")
                            info_href="https://weixin.sogou.com"+info_href
                            # TODO:需要获取永久链接
                        except:
                            info_href = ""
                            pass
                        each_data_dict = {}
                        each_data_dict["标题"] = info_title
                        each_data_dict["来源"] = info_source
                        each_data_dict["时间"] = info_time
                        each_data_dict["简介"] = info_breif
                        each_data_dict["链接"] = info_href
                        crawler_data_list.append(each_data_dict)

                        each_data_dict_json = {}
                        each_data_dict_json["title"] = str(info_title)
                        each_data_dict_json["source"] = info_source
                        each_data_dict_json["time"] = info_time
                        each_data_dict_json["introduction"] = info_breif
                        each_data_dict_json["link"] = info_href
                        each_data_dict_json["createTm"] = now_time
                        each_data_dict_json["taskId"] = taskId
                        crawler_data_list_json.append(each_data_dict_json)

                    data["crawler_data_list_json"] = crawler_data_list_json
                    data["crawler_data_list"] = crawler_data_list
            except Exception as exp:
                # 通过没有下一页的提示错误来分析，是否到了最后一页
                if "Cannot read property 'click' of null" in str(exp):
                    print("当前已到最后一页")
                else:
                    print("翻页错误：{}".format(exp))
                is_not_last_page = False
                pass
        dr.quit()
        return data

    # 获取翻页的下一页的按钮,如果没有为None
    def get_next_page_togger(self, web_driver):
        next_page_dom_tag = ["a", "span"]
        next_page_text = ["下一页", "下页"]
        next_page_element = None
        is_not_find_next = True
        for tag in next_page_dom_tag:
            for txt in next_page_text:
                next_page_path = "//{}[contains(text(),'{}')]".format(tag, txt)
                try:
                    next_page_element = web_driver.find_element_by_xpath(next_page_path)
                    is_not_find_next = False
                    break
                except Exception as e:
                    pass
                continue
            if is_not_find_next == False:
                break
        return next_page_element



"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: weiboCrawler.py
@Date: Create in 2021/1/25 16:19
@Description: 对微博数据进行抓取
@ Software: PyCharm
===================================
"""

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.keys import Keys
from time import sleep
from bs4 import BeautifulSoup
import re
from vgis_utils.vgis_datetime.datetimeTools import DateTimeHelper
from weiboSettings import Setting
import binascii


class CrawerWeiboData:

    # 初始化
    def __init__(self, query_word=None, turn_page=None):
        self.query_word = query_word
        self.turn_page = turn_page

    # 获取微博综合查询结果数据
    # https: // s.weibo.com / weibo?q = 森林火灾 & Refer = weibo_weibo & page = 2
    #  &xsort=hot  热门  &scope=ori 原创 &atten=1 关注人 &vip=1 认证用户 &category=4 媒体  &viewpoint=1观点
    # 时间范围： &timescope=custom:2021-01-04:2021-01-12
    #  含图片 &haspic=1  含视频 &hasvideo=1 含音乐 &hasmusic=1  含短链接 &haslink=1
    def get_Weibo_Compre_Query_Result(self):
        chrome_options = Options()  # 实例化Option对象
        chrome_options.add_argument('--headless')  # 把Chrome浏览器设置为静默模式
        chrome_options.add_argument('--disable-gpu')  # 禁止加载图片
        dr = webdriver.Chrome(options=chrome_options)  # 设置引擎为Chrome，在后台默默运行
        # 通过新浪邮箱登录，微博登录需要短信验证码
        URL = 'https://mail.sina.com.cn/?from=mail'
        dr.get(URL)
        sleep(1)
        # binascii.a2b_hex(auth_str).decode("utf-8")
        setting = Setting()
        auth_str = setting.get_AUTH_STR()
        decode_str = binascii.a2b_hex(auth_str).decode("utf-8")
        key_list = decode_str.split(setting.get_AUTH_SPLIT_STR())
        dr.find_element_by_xpath('//*[@id="freename"]').send_keys(key_list[0])
        dr.find_element_by_xpath('//*[@id="freepassword"]').send_keys(key_list[1])
        is_login_sina_email = True
        crawler_data_list = []
        try:
            # 自动获取登录按钮,还有问题
            # login_button_element = self.get_login_button_togger(dr)
            # dr.execute_script("arguments[0].click();", login_button_element)
            # 有广告的页面
            locator = '/html/body/div[3]/div/div[2]/div/div/div[4]/div[1]/div[1]/div[7]/div[1]/a[1]'
            if dr.find_element_by_xpath(locator) is not None:
                dr.find_element_by_xpath(locator).click()
            else:
                # 没有广告的页面
                locator = '/html/body/div[1]/div/div[2]/div/div/div[4]/div[1]/div[1]/div[7]/div[1]/a[1]'
                if dr.find_element_by_xpath(locator) is not None:
                    dr.find_element_by_xpath(locator).click()
        except:
            is_login_sina_email = False
            pass
        if is_login_sina_email:
            print("新浪邮箱登录成功，在后台浏览器打开新TAB页，进行微博免登录搜索，支持多页查询")
            # print(dr.current_url)
            # print(dr.page_source)
            print("----------------------------------------------------------")
            sleep(3)
            # 在浏览器打开 新tab，进行微博查询,利用新浪邮箱登录的身份，微博不用再登录
            dr.find_element_by_tag_name('body').send_keys(Keys.CONTROL + 't')
            is_not_last_page = True
            curent_page_num = 1
            while is_not_last_page and curent_page_num <= self.turn_page:
                try:
                    if curent_page_num == 1:
                        url = "https://s.weibo.com/weibo?q=" + self.query_word + "&Refer=weibo_weibo"
                        print("目前是第1页------")
                        dr.get(url)
                    else:
                        next_page_element = self.get_next_page_togger(dr)
                        dr.execute_script("arguments[0].click();", next_page_element)
                        sleep(1)
                        print("自动获取到第{}页-----".format(str(curent_page_num)))
                    # print(dr.page_source)
                    curent_page_num += 1
                    print("获取微博搜索数据.........")
                    htmlinfo = bytes(dr.page_source, encoding='utf-8')
                    soup = BeautifulSoup(htmlinfo, 'html.parser')
                    div_box = soup.find_all('div', attrs={'class': 'm-con-l'})
                    if len(div_box) > 0:
                        # card_item_list = div_box[0].find_all(name='div')[3].find_all('div',
                        #                                                              attrs={'class': 'card-wrap'})
                        card_item_list = div_box[0].find_all('div', attrs={'class': 'card-wrap'})
                        for card_item in card_item_list:
                            # 标题,提取简介里的 【】，若没有则提取前10个字符
                            try:
                                info_title_old = card_item.find_all('p', attrs={'class': 'txt'})[0].text.replace(
                                    "\n", "").strip()
                                info_title = re.findall("【(.*?)】", info_title_old)
                                if len(info_title) == 0:
                                    info_title = info_title_old[0:20]
                            except:
                                info_title = ""
                                pass
                            # 来源
                            try:
                                info_source = \
                                    card_item.find_all('div', attrs={'class': 'info'})[0].find_all(name='div')[
                                        1].text.replace("\n", "")
                            except:
                                info_source = ""
                                pass
                            # 时间
                            try:
                                info_time = \
                                    card_item.find_all('p', attrs={'class': 'from'})[0].text.replace("\n",
                                                                                                     "").strip().split(
                                        "\xa0")[0]
                                info_time = DateTimeHelper.get_standTime_By_Str(info_time)
                            except:
                                info_time = ""
                                pass
                            # 简介
                            try:
                                info_breif = card_item.find_all('p', attrs={'class': 'txt'})[0].text.replace("\n",
                                                                                                             "").strip()
                            except:
                                info_breif = ""
                                pass
                            # 链接
                            try:
                                info_href = "https:" + \
                                            card_item.find_all('p', attrs={'class': 'from'})[0].find_all(name='a')[
                                                0].attrs.get("href")
                            except:
                                info_href = ""
                                pass
                            each_data_dict = {}
                            each_data_dict["标题"] = info_title
                            each_data_dict["来源"] = info_source
                            each_data_dict["时间"] = info_time
                            each_data_dict["简介"] = info_breif
                            each_data_dict["链接"] = info_href
                            crawler_data_list.append(each_data_dict)
                except Exception as exp:
                    # 通过没有下一页的提示错误来分析，是否到了最后一页
                    if "Cannot read property 'click' of null" in str(exp):
                        print("当前已到最后一页")
                    else:
                        print("翻页错误：{}".format(exp))
                    is_not_last_page = False
                    pass
        dr.quit()
        return crawler_data_list

    # 获取微博综合查询结果数据
    # https: // s.weibo.com / weibo?q = 森林火灾 & Refer = weibo_weibo & page = 2
    #  &xsort=hot  热门  &scope=ori 原创 &atten=1 关注人 &vip=1 认证用户 &category=4 媒体  &viewpoint=1观点
    # 时间范围： &timescope=custom:2021-01-04:2021-01-12
    #  含图片 &haspic=1  含视频 &hasvideo=1 含音乐 &hasmusic=1  含短链接 &haslink=1
    def get_Weibo_Compre_Query_Result_Json(self,now_time,taskId):
        chrome_options = Options()  # 实例化Option对象
        chrome_options.add_argument('--headless')  # 把Chrome浏览器设置为静默模式
        chrome_options.add_argument('--disable-gpu')  # 禁止加载图片
        dr = webdriver.Chrome(options=chrome_options)  # 设置引擎为Chrome，在后台默默运行
        # 通过新浪邮箱登录，微博登录需要短信验证码
        URL = 'https://mail.sina.com.cn/?from=mail'
        dr.get(URL)
        sleep(1)
        # binascii.a2b_hex(auth_str).decode("utf-8")
        setting = Setting()
        auth_str = setting.get_AUTH_STR()
        decode_str = binascii.a2b_hex(auth_str).decode("utf-8")
        key_list = decode_str.split(setting.get_AUTH_SPLIT_STR())
        dr.find_element_by_xpath('//*[@id="freename"]').send_keys(key_list[0])
        dr.find_element_by_xpath('//*[@id="freepassword"]').send_keys(key_list[1])
        is_login_sina_email = True
        crawler_data_list = []
        crawler_data_list_json = []
        data = {}
        try:
            # 自动获取登录按钮,还有问题
            # login_button_element = self.get_login_button_togger(dr)
            # dr.execute_script("arguments[0].click();", login_button_element)
            # 有广告的页面
            locator = '/html/body/div[3]/div/div[2]/div/div/div[4]/div[1]/div[1]/div[7]/div[1]/a[1]'
            if dr.find_element_by_xpath(locator) is not None:
                dr.find_element_by_xpath(locator).click()
            else:
                # 没有广告的页面
                locator = '/html/body/div[1]/div/div[2]/div/div/div[4]/div[1]/div[1]/div[7]/div[1]/a[1]'
                if dr.find_element_by_xpath(locator) is not None:
                    dr.find_element_by_xpath(locator).click()
        except:
            is_login_sina_email = False
            pass
        if is_login_sina_email:
            print("新浪邮箱登录成功，在后台浏览器打开新TAB页，进行微博免登录搜索，支持多页查询")
            # print(dr.current_url)
            # print(dr.page_source)
            print("----------------------------------------------------------")
            sleep(3)
            # 在浏览器打开 新tab，进行微博查询,利用新浪邮箱登录的身份，微博不用再登录
            dr.find_element_by_tag_name('body').send_keys(Keys.CONTROL + 't')
            is_not_last_page = True
            curent_page_num = 1
            while is_not_last_page and curent_page_num <= self.turn_page:
                try:
                    if curent_page_num == 1:
                        url = "https://s.weibo.com/weibo?q=" + self.query_word + "&Refer=weibo_weibo"
                        print("目前是第1页------")
                        dr.get(url)
                    else:
                        next_page_element = self.get_next_page_togger(dr)
                        dr.execute_script("arguments[0].click();", next_page_element)
                        sleep(1)
                        print("自动获取到第{}页-----".format(str(curent_page_num)))
                    # print(dr.page_source)
                    curent_page_num += 1
                    print("获取微博搜索数据.........")
                    htmlinfo = bytes(dr.page_source, encoding='utf-8')
                    soup = BeautifulSoup(htmlinfo, 'html.parser')
                    div_box = soup.find_all('div', attrs={'class': 'm-con-l'})
                    if len(div_box) > 0:
                        # card_item_list = div_box[0].find_all(name='div')[3].find_all('div',
                        #                                                              attrs={'class': 'card-wrap'})
                        card_item_list = div_box[0].find_all('div', attrs={'class': 'card-wrap'})
                        for card_item in card_item_list:
                            # 标题,提取简介里的 【】，若没有则提取前10个字符
                            try:
                                info_title_old = card_item.find_all('p', attrs={'class': 'txt'})[0].text.replace(
                                    "\n", "").strip()
                                info_title = re.findall("【(.*?)】", info_title_old)
                                if len(info_title) == 0:
                                    info_title = info_title_old[0:20]
                            except:
                                info_title = ""
                                pass
                            # 来源
                            try:
                                info_source = \
                                    card_item.find_all('div', attrs={'class': 'info'})[0].find_all(name='div')[
                                        1].text.replace("\n", "")
                            except:
                                info_source = ""
                                pass
                            # 时间
                            try:
                                info_time = \
                                    card_item.find_all('p', attrs={'class': 'from'})[0].text.replace("\n",
                                                                                                     "").strip().split(
                                        "\xa0")[0]
                                info_time = DateTimeHelper.get_standTime_By_Str(info_time)
                            except:
                                info_time = ""
                                pass
                            # 简介
                            try:
                                info_breif = card_item.find_all('p', attrs={'class': 'txt'})[0].text.replace("\n",
                                                                                                             "").strip()
                            except:
                                info_breif = ""
                                pass
                            # 链接
                            try:
                                info_href = "https:" + \
                                            card_item.find_all('p', attrs={'class': 'from'})[0].find_all(name='a')[
                                                0].attrs.get("href")
                            except:
                                info_href = ""
                                pass
                            each_data_dict = {}
                            each_data_dict["标题"] = info_title
                            each_data_dict["来源"] = info_source
                            each_data_dict["时间"] = info_time
                            each_data_dict["简介"] = info_breif
                            each_data_dict["链接"] = info_href
                            each_data_dict_json = {}
                            each_data_dict_json["title"] = str(info_title)
                            each_data_dict_json["source"] = info_source
                            each_data_dict_json["time"] = info_time
                            each_data_dict_json["introduction"] = info_breif
                            each_data_dict_json["link"] = info_href
                            each_data_dict_json["createTm"] = now_time
                            each_data_dict_json["taskId"] = taskId
                            crawler_data_list_json.append(each_data_dict_json)
                            crawler_data_list.append(each_data_dict)
                        data["crawler_data_list"] = crawler_data_list
                        data["crawler_data_list_json"] = crawler_data_list_json

                except Exception as exp:
                    # 通过没有下一页的提示错误来分析，是否到了最后一页
                    if "Cannot read property 'click' of null" in str(exp):
                        print("当前已到最后一页")
                    else:
                        print("翻页错误：{}".format(exp))
                    is_not_last_page = False
                    pass
        dr.quit()
        return data

    # 获取翻页的下一页的按钮,如果没有为None
    def get_next_page_togger(self, web_driver):
        next_page_dom_tag = ["a", "span"]
        next_page_text = ["下一页", "下页"]
        next_page_element = None
        is_not_find_next = True
        for tag in next_page_dom_tag:
            for txt in next_page_text:
                next_page_path = "//{}[contains(text(),'{}')]".format(tag, txt)
                try:
                    next_page_element = web_driver.find_element_by_xpath(next_page_path)
                    is_not_find_next = False
                    break
                except Exception as e:
                    pass
                continue
            if is_not_find_next == False:
                break
        return next_page_element

    # 获取登录的按钮,如果没有为None
    def get_login_button_togger(self, web_driver):
        login_button_tag = ["a", "span"]
        login_button_text = ["登录"]
        login_button_element = None
        is_not_find_login = True
        for tag in login_button_tag:
            for txt in login_button_text:
                login_button_path = "//{}[contains(text(),'{}')]".format(tag, txt)
                try:
                    login_button_element = web_driver.find_element_by_xpath(login_button_path)
                    is_not_find_login = False
                    break
                except Exception as e:
                    pass
                continue
            if is_not_find_login == False:
                break
        return login_button_element


#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    : 2021/2/1 10:28
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : spiderTest.py
# @Desc    ：抓取程序测试帮助类
# @Software: PyCharm

import binascii


class TestHelper:
    # 初始化
    def __init__(self):
        pass

    # 构建识别网址命令
    def build_parse_website_command(self, web_site, excel_path):
        web_site = str(binascii.b2a_hex(web_site.encode())).lstrip("b'").rstrip("'")
        command_str = "python parseWebsiteData.py -w {} -d {}".format(web_site, excel_path)
        return command_str

    # 构建抓取网址命令
    def build_crawler_website_command(self, web_site, field_name_str, field_path_str, is_turn_page, turn_page_num,
                                      excel_path):
        web_site = str(binascii.b2a_hex(web_site.encode())).lstrip("b'").rstrip("'")
        field_path_str = str(binascii.b2a_hex(field_path_str.encode())).lstrip("b'").rstrip("'")
        command_str = "python crawlerWebsiteData.py -w {} -t {} -s {} -n {} -p {} -d {}".format(web_site,
                                                                                                str(is_turn_page),
                                                                                                str(turn_page_num),
                                                                                                field_name_str,
                                                                                                field_path_str,
                                                                                                excel_path)
        return command_str



"""
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Project :pythonCodeSnippet
@File    :downloadTilesTools.py
@IDE     :PyCharm
@Author  :chenxw
@Date    :2023/11/2 13:52
@Descr:
"""
import json
import os
import shutil
import uuid
from datetime import datetime
from math import floor

import cv2
import numpy as np
import requests as req
from PIL import Image
from loguru import logger
from vgis_gis.geoserverTools import GeoserverOperatoer

# 在线地图服务瓦片下载帮助类
class downloadTilesHelper:

    # 构造函数
    def __init__(self, tile_pic_suffix, merge_pic_name, resolution, tile_size, image_service_info, image_service_type,
                 tile_pic_path,
                 vcontat_img_path,
                 out_image_path):
        # 瓦片图片后缀
        self.tile_pic_suffix = tile_pic_suffix
        # 合成图片名称
        self.merge_pic_name = merge_pic_name
        # 分辨率，每像素对应的地图单位值，值越小，精度越高，合成图片越大
        self.resolution = resolution
        # 下载瓦片像素大小
        self.tile_size = tile_size
        # 要下载瓦片的影像服务信息
        self.image_service_info = image_service_info
        # 要下载瓦片的影像服务类型，如wms,wmts,tms,xyz等
        self.image_service_type = image_service_type
        # 对于非wms服务，需要重新计算分辨率和瓦片像素大小
        if image_service_type != "wms" and "geoserver" in image_service_info["service_type"]:
            self.resolution, self.tile_size, self.tile_coords = self.get_details_of_zoomlevel_in_geoserver(
                self.image_service_info["zoom_level"], self.image_service_info["gridset_name"])

        # 瓦片存储路径
        self.tile_pic_path = tile_pic_path if tile_pic_path is not None else os.path.join(os.getcwd(), ".cache",
                                                                                          "allTiles")
        # 每列瓦片拼接的竖条图片存放路径
        self.vcontat_img_path = vcontat_img_path if vcontat_img_path is not None else os.path.join(os.getcwd(),
                                                                                                   ".cache", "vMTiles")
        # 所有瓦片合成大图存放路径
        self.out_image_path = out_image_path
        self.retry_download_curent = 0
        self.retry_download_count = 5
        self.delete_cache()
        self.mkdir(self.tile_pic_path)
        self.mkdir(self.vcontat_img_path)
        self.mkdir(self.out_image_path)

    # 获取geoserver切片里指定分级对应的分辨率
    def get_details_of_zoomlevel_in_geoserver(self, zoom_level, grid_set_name):
        geoserverOperatoer = GeoserverOperatoer(self.image_service_info["service_url"].split("/gwc/")[0],
                                                self.image_service_info["geoserver_username"],
                                                self.image_service_info["geoserver_password"])
        gridset_info = geoserverOperatoer.get_grid_set_info(grid_set_name)
        gridset_json = json.loads(gridset_info)
        return gridset_json["gridSet"]["resolutions"][zoom_level], gridset_json["gridSet"]["tileWidth"], \
            gridset_json["gridSet"]["extent"]["coords"]

    # 根据坐标范围计算WMTS/TMS瓦片的行列号
    def compute_col_row_by_coords(self, xmin, xmax, ymin, ymax):
        tileMinCol, tileMaxCol, tileMinRow, tileMaxRow = None, None, None, None
        image_service_type = self.image_service_type
        # 为补偿浮点计算的不准确性
        epsilon = 1e-6
        # 左上角为原点
        if image_service_type == "wmts" or image_service_type == "xyz":
            if self.image_service_info["service_type"] == "geoserver_wmts":
                # 原点（左上角）
                tileMatrixMinx = self.tile_coords[0]
                tileMatrixMaxY = self.tile_coords[3]
                tileMinCol = round(floor(abs(xmin - tileMatrixMinx) / self.resolution / self.tile_size + epsilon))
                tileMaxCol = round(floor(abs(xmax - tileMatrixMinx) / self.resolution / self.tile_size - epsilon))
                tileMinRow = round(floor(abs(tileMatrixMaxY - ymax) / self.resolution / self.tile_size + epsilon))
                tileMaxRow = round(floor(abs((tileMatrixMaxY - ymin)) / self.resolution / self.tile_size - epsilon))
        # 左下角为原点
        elif image_service_type == "tms":
            if self.image_service_info["service_type"] == "geoserver_tms":
                tileMatrixMinx = self.tile_coords[0]
                tileMatrixMiny = self.tile_coords[1]
                tileMinCol = round(floor(abs(xmin - tileMatrixMinx) / self.resolution / self.tile_size + epsilon))
                tileMaxCol = round(floor(abs(xmax - tileMatrixMinx) / self.resolution / self.tile_size - epsilon))
                tileMinRow = round(floor(abs(ymax - tileMatrixMiny) / self.resolution / self.tile_size + epsilon))
                tileMaxRow = round(floor(abs(ymin - tileMatrixMiny) / self.resolution / self.tile_size - epsilon))
        matrixWidth = pow(2, self.image_service_info["zoom_level"] + 1)
        matrixHeight = pow(2, self.image_service_info["zoom_level"])
        if tileMinCol < 0:
            tileMinCol = 0
        if tileMaxCol >= matrixWidth:
            tileMaxCol = matrixWidth - 1
        if tileMinRow < 0:
            tileMinRow = 0
        if tileMaxRow >= matrixHeight:
            tileMaxRow = matrixHeight - 1
        return tileMinCol, tileMaxCol, tileMinRow, tileMaxRow

    # 下载wmts/tms/xyz瓦片
    def loop_download_tiles(self, xmin, xmax, ymin, ymax):
        tileMinCol, tileMaxCol, tileMinRow, tileMaxRow = self.compute_col_row_by_coords(xmin, xmax, ymin, ymax)
        if self.image_service_type == "wmts":
            for col in range(tileMinCol, tileMaxCol + 1):
                for row in range(tileMinRow, tileMaxRow + 1):
                    tile_url = self.build_wmts_tile_pic_url(col, row, self.image_service_info["zoom_level"])
                    # 下载瓦片
                    self.retry_download_curent = 0
                    self.download_each_tile(tile_url, "row=" + str(row) + "&col=" + str(col), col)
            # 同时计算左上角瓦片的地图单位
            # 瓦片原点在左上角
            left_top_tile_bounds = self.get_bounds_of_geoserver_wmts_tile(self.image_service_info["zoom_level"],
                                                                          tileMinRow, tileMinCol)

        elif self.image_service_type == "tms":
            pass
            # 同时计算左上角瓦片的地图单位
            # 瓦片原点在左下角
            left_top_tile_bounds = self.get_bounds_of_geoserver_wmts_tile(self.image_service_info["zoom_level"],
                                                                          tileMaxRow,
                                                                          tileMinCol)
        elif self.image_service_type == "xyz":
            pass
            # 同时计算左上角瓦片的地图单位
            # 瓦片原点在左上角
            left_top_tile_bounds = self.get_bounds_of_geoserver_wmts_tile(self.image_service_info["zoom_level"],
                                                                          tileMinRow,
                                                                          tileMinCol)
        return left_top_tile_bounds

    # 获取瓦片的地图坐标范围
    def get_bounds_of_geoserver_wmts_tile(self, level, row, col):
        tile_url = self.build_wmts_tile_pic_url(col, row, level)
        payload = {}
        headers = {}
        response = req.request("GET", tile_url, headers=headers, data=payload)
        headers = response.headers
        tile_bounds = headers["geowebcache-tile-bounds"]
        return tile_bounds.split(",")

    # 按照指定范围下载瓦片并拼接成大图
    def fetch_tiles(self, xmin, xmax, ymin, ymax):
        try:
            # 对影像服务的指定区域进行瓦片下载（针对下载不成功的进行重试），拼接成jpg/png，并生成jgw/pgw
            # 如果已经下载则跳过
            # 目录及文件命名形式：
            left_x = xmin
            top_y = ymax

            # 先判断缓存里有没有合成的图片数据，没有再去重新下载
            if not os.path.exists(os.path.join(self.out_image_path, self.merge_pic_name)):
                if self.image_service_type == "wms":
                    # 按照每个瓦片的大小作为步长进行下载
                    # 沿着X方向,逐步长滑动
                    while xmin < xmax:
                        self.loop_download_wms_tiles_follow_y(xmin, ymin, ymax)
                        xmin += (self.resolution * self.tile_size)
                elif self.image_service_type == "wmts" or self.image_service_type == "tms" or self.image_service_type == "xyz":
                    left_top_tile_bounds = self.loop_download_tiles(xmin, xmax, ymin, ymax)

                logger.info('瓦片下载完成')

                # 拼接瓦片图片
                logger.info('正在拼接瓦片为图片')
                try:
                    self.merge_tiles_to_image()
                    # 根据坐标范围反算瓦片的行列号，最后拼成的瓦片大图，比输入的坐标范围要大些，因为需要重新计算左上角点的地图坐标
                    if self.image_service_type != "wms":
                        left_x, top_y = float(left_top_tile_bounds[0]), float(left_top_tile_bounds[3])
                    # 生成world文件
                    self.build_merge_image_world_file_info(left_x, top_y)
                except Exception as tm_exp:
                    logger.error("瓦片拼接失败，请检查瓦片：{}".format(str(tm_exp)))
                    logger.error(tm_exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
                    logger.error(tm_exp.__traceback__.tb_lineno)  # 发生异常所在的行数
            else:
                logger.warning("之前已拼接过图片")
            self.delete_cache()
        except Exception as exp:
            logger.error("下载瓦片失败：{}".format(str(exp)))
            logger.error(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            logger.error(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

    def delete_cache(self):
        # 清空下载的瓦片缓存（中间数据）
        if os.path.exists(self.tile_pic_path):
            shutil.rmtree(self.tile_pic_path)
        # 清空拼接的竖条瓦片图片（中间数据）
        if os.path.exists(self.vcontat_img_path):
            shutil.rmtree(self.vcontat_img_path)

    # 将瓦片合并成大图
    def merge_tiles_to_image(self):
        #  瓦片数据目录组织，是按照每个下载步长
        tiles_stride_dirs = os.listdir(self.tile_pic_path)
        # 先生成所有的竖条
        for tile_stride_dir in tiles_stride_dirs:
            self.vconcat_tileImage(tile_stride_dir)
        # 对所有竖条进行拼接成最终大图
        final, width, height = self.hconcat_vconcatImages(self.vcontat_img_path)
        logger.info('瓦片拼接完成')
        final.save(os.path.join(self.out_image_path, self.merge_pic_name))

    # 生成每一列的图片,即竖条
    def vconcat_tileImage(self, tile_stride_dir):
        tile_img_dir = os.path.join(self.tile_pic_path, tile_stride_dir)  # 原始文件目录
        names = os.listdir(tile_img_dir)
        #
        if self.image_service_type == "wms":
            # 竖着的瓦片要从上往下粘贴，对应的图片名称，Y的值越大越往上，所以做了反转
            # names.reverse()
            names = sorted(names, reverse=True)
            # sorted_names = sorted(names, key=self.custom_sort, reverse=True)
        elif self.image_service_type == "wmts":
            # wmts的原点在左下方，因此COL值越小越往上，不做反转
            names = sorted(names)

        images = []
        for name in names:
            img_path = os.path.join(tile_img_dir, name)
            # 拼接图片不稳定，如果图片中只有少量，就会报错
            # error: (-215:Assertion failed) src[i].dims <= 2 && src[i].cols == src[0].cols && src[i].type() == src[0].type() in function 'cv::vconcat'
            # 不完整的瓦片多了一个通道，需要删除，导致拼接的图片shape不一致,正确图片shape为(256,256,3) 错误瓦片shape为(256,256,4)
            # 修改by chenxw 将参数里的-1改成了1，有些不完整的瓦片通道变成了四个，需要这里只读前三个通道，和完整瓦片的三通道保持一致
            image = cv2.imdecode(np.fromfile(img_path, dtype=np.uint8), 1)
            images.append(image)
        img = cv2.vconcat(images)
        compress_rate = 1
        height, width = img.shape[:2]
        # 双三次插值
        img_resize = cv2.resize(img, (int(width * compress_rate), int(height * compress_rate)),
                                interpolation=cv2.INTER_AREA)
        # 保存每列瓦片竖条合成图
        vconcat_pic_path = os.path.join(self.vcontat_img_path, '{}.{}'.format(tile_stride_dir, self.tile_pic_suffix))
        # 解决中文路径保存问题
        cv2.imencode('.{}'.format(self.tile_pic_suffix), img_resize)[1].tofile(vconcat_pic_path)
        cv2.waitKey(0)
        cv2.destroyAllWindows()

    # 对所有每一列竖条图片进行合并生成大图
    def hconcat_vconcatImages(self, img_dir):
        images = os.listdir(img_dir)
        # 每个竖条图片名称为x值，或者col，从小到大，依次对应大图的从左到右
        images = sorted(images)

        # 只需要图片,过滤掉目录里非图片
        def file_filter(f):
            if os.path.splitext(f)[1] in ['.jpg', '.jpeg', '.png', '.bmp']:
                return True
            else:
                return False

        images = list(filter(file_filter, images))
        img = Image.open(os.path.join(img_dir, images[0]))
        width = img.size[0] * len(images)
        height = img.size[1]
        final = Image.new('RGB', (width, height))

        for index, image in enumerate(images):
            img = Image.open(os.path.join(img_dir, image))
            final.paste(img, ((img.size[0] * index), 0))

        return final, width, height

    # 生成jgw/pgw --world file世界投影文件
    def build_merge_image_world_file_info(self, left_x, top_y):
        world_file_suffix = "jgw"
        if self.tile_pic_suffix == "jpg" or self.tile_pic_suffix == "jpeg":
            world_file_suffix = "jgw"
        elif self.tile_pic_suffix == "png":
            world_file_suffix = "pgw"
        file_pre_path, file_name = os.path.split(self.merge_pic_name)
        shotname, suffix = os.path.splitext(file_name)
        filename = os.path.join(self.out_image_path, "{}.{}".format(shotname, world_file_suffix))
        with open(filename, 'w') as file:
            # 在X方向的比例关系;每像素对应的经度值
            file.write("{}\n".format(self.resolution))
            # 影像数据的旋转参数，一般忽略，记录为0
            file.write("0.0000000000\n")
            # 影像数据的旋转参数，一般忽略，记录为0
            file.write("0.0000000000\n")
            # 在Y方向的比例关系;;每像素对应的纬度值，为负值, 这是由于空间坐标系与影像图片像素坐标系在Y方向上相反
            file.write("-{}\n".format(self.resolution))
            # 影像图片左上角点对应的空间坐标的X坐标
            file.write(str(left_x) + "\n")
            # 影像图片左上角点对应的空间坐标的Y坐标
            file.write(str(top_y) + "\n")

    # 沿着Y方向循环下载wms瓦片
    # 方向为从上往下
    def loop_download_wms_tiles_follow_y(self, xmin, ymin, ymax):
        # 沿着Y方向
        while ymax > ymin:
            bbox = str(xmin) + "%2C" + str(ymax - (self.resolution * self.tile_size)) + "%2C" + str(
                xmin + (self.resolution * self.tile_size)) + "%2C" + str(ymax)
            tile_url = self.build_wms_tile_pic_url(bbox)
            # 下载瓦片
            self.retry_download_curent = 0
            self.download_each_tile(tile_url, "y=" + str(ymax) + "&x=" + str(xmin), xmin)
            ymax -= (self.resolution * self.tile_size)

    # 获取wmts瓦片下载地址
    def build_wmts_tile_pic_url(self, col, row, level):
        url = None
        if self.image_service_info["service_type"] == "geoserver_wmts":
            url = "{}layer={}&style=&tilematrixset={}&Service=WMTS&Request=GetTile&Version=1.0.0&Format=image/{}&TileMatrix={}:{}&TileCol={}&TileRow={}".format(
                self.image_service_info["service_url"], self.image_service_info["layer_name"],
                self.image_service_info["gridset_name"], self.tile_pic_suffix, self.image_service_info["gridset_name"],
                level, col, row)
        else:
            pass
        return url

    # 构造下载的wms地图服务影像图片下载URL
    def build_wms_tile_pic_url(self, bbox):
        # geoserver发布的wms服务
        if self.image_service_info["service_type"] == "geoserver_wms":
            # url里的中文编码问题
            # layer_name = urllib.parse.quote(self.image_service_info["layer_name"].encode("gb2312"))
            data = {
                'LAYERS': self.image_service_info["layer_name"],
                'SERVICE': 'WMS',
                'REQUEST': 'GetMap',
                'SRS': 'EPSG:4326',
                'WIDTH': '{}'.format(self.tile_size),
                'HEIGHT': '{}'.format(self.tile_size),
                'STYLES': '',
                'BBOX': bbox,
                'FORMAT': "image/{}".format(self.tile_pic_suffix)
            }
            # 拼接url参数
            lt = []
            for k, v in data.items():
                lt.append(k + '=' + str(v))
            # 拼接url
            url = self.image_service_info["service_url"] + '&'.join(lt)
        # 卫星中心一张图2米WMS服务
        if self.image_service_info["service_type"] == "satellite_center_wms":
            data = {
                'SERVICE': 'WMS',
                'REQUEST': 'GetMap',
                # 'SRS': 'EPSG%3A4326',
                'SRS': 'WGS84',
                'ACCOUNT': 'zkzy',
                'PASSWD': 'fee528666a542aeb9f086093883c453d',
                'WIDTH': '{}'.format(self.tile_size),
                'HEIGHT': '{}'.format(self.tile_size),
                'FORMAT': 'image/{}'.format(self.tile_pic_suffix),
                'STYLES': '',
                'BBOX': bbox
            }
            # 拼接url参数
            lt = []
            for k, v in data.items():
                lt.append(k + '=' + str(v))
            # 拼接url
            url = self.image_service_info["service_url"] + "/OneMap" + self.image_service_info[
                "layer_name"] + "/wms?" + '&'.join(lt)

        # logger.info("瓦片地址：" + url)
        return url

    # 下载每个小瓦片
    def download_each_tile(self, url, pic_name, pic_dir_name):
        if self.retry_download_curent <= 5:
            try:
                if self.retry_download_curent > 0:
                    logger.info("self.retry_download_curent:{}".format(self.retry_download_curent))
                self.retry_download_curent += 1
                # 读取远程图片
                path = os.path.join(self.tile_pic_path, str(pic_dir_name))
                if not os.path.exists(path):
                    # 创建文件夹
                    self.mkdir(path)
                    logger.info("创建文件夹:" + path + "--" + datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                tilefile = os.path.join(path, '{}.{}'.format(pic_name, self.tile_pic_suffix))
                # # 如果瓦片存在，但大于100K，说明有可能是损坏的瓦片，需要重新下载-修改chenxw
                # if os.path.exists(tilefile):
                #     fileSize = os.path.getsize(tilefile) / 1024
                #     if fileSize > 100:
                #         os.remove(tilefile)
                # 如果瓦片图片不存在才下载
                if not os.path.exists(tilefile):
                    # url中有中文问题
                    # pic = urllib.request.urlopen(url).read()
                    pic = req.get(url)
                    # 创建瓦片图片
                    with open(tilefile, 'wb') as f:
                        f.write(pic.content)
                # 如果已下载瓦片不完整，需要重新下载
                else:
                    # print("检查文件夹下的瓦片:" + path)
                    try:
                        img_cv = cv2.imdecode(np.fromfile(tilefile, dtype=np.uint8), -1)
                        if img_cv is not None:
                            if img_cv.shape[2] == 4:
                                logger.info(
                                    "下载瓦片不完整，需要重新下载：" + tilefile + "--" + datetime.now().strftime(
                                        '%Y-%m-%d %H:%M:%S'))
                                os.remove(tilefile)
                                self.download_each_tile(url, pic_name, pic_dir_name)
                            else:
                                pass
                                # logger.info("瓦片已存在且完整不再下载---" + datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                    except BaseException as ex:
                        logger.error(
                            "读取瓦片文件失败---" + tilefile + "---" + datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                        os.remove(tilefile)
                        # 瓦片下载失败，重试
                        self.download_each_tile(url, pic_name, pic_dir_name)
            except BaseException as ex:
                logger.error("下载失败" + str(ex) + "---" + datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                logger.error(url)
                if str(ex) != "HTTP Error 404: Not Found":
                    logger.error("正在重新下载---" + datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                    # 瓦片下载失败，重试
                    self.download_each_tile(url, pic_name, pic_dir_name)

    # 创建文件夹
    def mkdir(self, path):
        folder = os.path.exists(path)
        # 判断是否存在文件夹如果不存在则创建为文件夹
        if not folder:
            # 创建文件时如果路径不存在会创建这个路径
            os.makedirs(path)

    # 删除文件夹内文件
    def rmdir(self, img_dir):
        images = os.listdir(img_dir)
        for name in images:
            img_path = os.path.join(img_dir, name)
            os.remove(img_path)


#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    : 2021/1/7 23:44
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : htmlParse.py
# @Desc    ：
# @Software: PyCharm

# 网页自动解析内容需要过滤的文本
filterValueArray = [
"忘记密码",
"注册用户",
"我要提问",
"清空",
"反馈/投诉"

]


#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
from PIL import Image
import pytesseract


# https://www.jb51.net/article/183990.htm
# 针对没有干扰的图片文字能识别处理
# 但是不能识别有干扰的图片
def read_ch_text(text_path):
    """
    传入文本(jpg、png)的绝对路径,读取文本
    :param text_path:
    :return: 文本内容
    """
    # 验证码图片转字符串
    im = Image.open(text_path)
    # 转化为8bit的黑白图片
    imgry = im.convert('L')
    # 二值化，采用阈值分割算法，threshold为分割点
    threshold = 140
    table = []
    for j in range(256):
        if j < threshold:
            table.append(0)
        else:
            table.append(1)
    out = imgry.point(table, '1')
    # 识别文本，lang参数改为chi_sim，其他代码与上面的读取验证码代码一致。
    text = pytesseract.image_to_string(out, lang="chi_sim", config='--psm 6')
    return text


def read_en_text(text_path):
    """
    传入文本(jpg、png)的绝对路径,读取文本
    :param text_path:
    :return: 文本内容
    """
    # 验证码图片转字符串
    im = Image.open(text_path)
    # 转化为8bit的黑白图片
    imgry = im.convert('L')
    # 二值化，采用阈值分割算法，threshold为分割点
    threshold = 140
    table = []
    for j in range(256):
        if j < threshold:
            table.append(0)
        else:
            table.append(1)
    out = imgry.point(table, '1')
    # 识别文本
    text = pytesseract.image_to_string(out, lang="eng", config='--psm 6')
    return text


if __name__ == '__main__':
    print("图片1：" + read_en_text("E:\\desktop\\response.jpeg"))
    print("图片2：" + read_en_text("E:\\desktop\\wpvvgzigca.png"))
    print("图片3：" + read_en_text("E:\\desktop\\202004020914468.png"))
    print("图片4：" + read_ch_text("E:\\desktop\\2020040209144612.png"))
    print("图片5：" + read_ch_text("E:\\desktop\\2021-12-28_175243.png"))



#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    : 2021/1/7 23:44
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : htmlParse.py
# @Desc    ：对网页html进行解析，提取相关信息
# @Software: PyCharm
import time
import requests
from bs4 import BeautifulSoup
from bs4.element import Comment
from bs4.element import NavigableString
from selenium import webdriver
from selenium.webdriver.chrome.options import Options

from specialValue import filterValueArray
from vgis_utils.iteratorTools import IteratorHelper
from vgis_utils.vgis_string.stringTools import StringHelper

headers = {
    'User-Agent': 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E)',
    'Accept': 'image/webp,image/*,*/*;q=0.8',
    'Accept-Encoding': 'gzip, deflate',
    'Referer': 'http://www.baidu.com/link?url=_andhfsjjjKRgEWkj7i9cFmYYGsisrnm2A-TN3XZDQXxvGsM9k9ZZSnikW2Yds4s&amp;amp;wd=&amp;amp;eqid=c3435a7d00006bd600000003582bfd1f',
    'Connection': 'keep-alive'}


class WebDataParser:
    # 初始化
    def __init__(self, siteUrl=None):
        print("采集网址为{}".format(siteUrl))
        self.siteUrl = siteUrl
        self.is_daynamic_data = False
        self.web_driver = None

    # 设置网址
    def set_curent_page_url(self, web_url):
        self.siteUrl = web_url

    # 设置网页的html
    def set_curent_page_html(self, html_str):
        # self.htmlinfo = bytes(html_str, encoding='utf-8')
        self.htmlinfo = html_str

    # 网址html内容解析
    # TODO:切换代理，防止ip被封
    # # 每次连接等待几秒，避免出现验证码
    def get_web_page_html(self):
        r = requests.get(url=self.siteUrl, headers=headers)
        r.encoding = r.apparent_encoding
        self.htmlinfo = r.text
        # r.encoding = 'utf-8'
        # # r.encoding = 'gb2312'
        # self.htmlinfo = r.content
        if self.is_dynamic_content_by_javascrit(self.htmlinfo):
            print("是动态数据网址，切换到selenium模拟谷歌浏览器访问模式")
            self.is_daynamic_data = True
            chrome_options = Options()  # 实例化Option对象
            chrome_options.add_argument('--headless')  # 把Chrome浏览器设置为静默模式
            chrome_options.add_argument('--disable-gpu')  # 禁止加载图片
            # desired_capabilities = DesiredCapabilities.CHROME
            # desired_capabilities["pageLoadStrategy"] = "none"  # 设置非阻塞
            # driver = webdriver.Chrome(options=chrome_options,desired_capabilities=desired_capabilities)  # 设置引擎为Chrome，在后台默默运行
            driver = webdriver.Chrome(options=chrome_options)  # 设置引擎为Chrome，在后台默默运行
            self.web_driver = driver
            # driver.implicitly_wait(10)  #隐式等待
            driver.get(self.siteUrl)
            # 强制等待10秒，解决部分网站数据没有完全加载的问题
            time.sleep(10)
            tmp_htmlinfo = driver.page_source
            # tmp_htmlinfo = etree.HTML(driver.page_source)
            # self.htmlinfo = bytes(tmp_htmlinfo, encoding='utf-8')
            self.htmlinfo = bytes(tmp_htmlinfo, encoding=self.get_encode_in_html(tmp_htmlinfo))
            # driver.quit()
            # driver.close()
        # return htmlinfo

    def get_encode_in_html(self, htmlcode):
        # htmlcode = "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=gb2312\"><html>"
        # htmlcode = "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gbk\" /><html>"
        # htmlcode = "<meta charset=\"gb2312\"><html>"
        # htmlcode = "<meta charset=\"utf-8\" /><html>"
        encode = StringHelper.get_str_btw(htmlcode, "charset=", ">").replace("\"", "").replace("/", "").strip()
        return encode

    # 判断是否为动态脚本生成的网页内容
    def is_dynamic_content_by_javascrit(self, htmlinfo):
        is_dynamic = False
        soup = BeautifulSoup(htmlinfo, 'html.parser')
        div_text = ""
        for div in soup.find_all(name='div'):
            div_text += div.text.strip()
        if div_text == "" or "加载" in div_text or "Loading" in div_text or "loading" in div_text:
            is_dynamic = True
        return is_dynamic

    # 得到网址数据是否动态数据
    def get_web_page_data_type(self):
        return self.is_daynamic_data

    # 自动解析网页中的ul列表数据
    def parse_ul_in_html(self):
        self.get_web_page_html()
        soup = BeautifulSoup(self.htmlinfo, 'html.parser')
        # 所有识别结果的数据记录数组
        all_recog_data_record_list = []
        # 所有识别结果的字段信息数组（字段名，字段路径）
        all_recog_data_field_list = []
        for ul in soup.find_all(name='ul'):
            # ul里文本不能为空，或者换行符
            if ul.text.strip() != "" and ul.text.strip() != "\n":
                # ul下的li数量要超过1个
                if len(ul.find_all(name='li')) > 1:
                    # li的子元素个数大于1
                    iterator_operator = IteratorHelper(ul.find(name='li').children)
                    iterator_count = iterator_operator.get_iterator_count()
                    if iterator_count > 1:
                        # li的有效字段个数，即text内容有效值数量超过1个
                        li_valid_filed_count = 0
                        for li in ul.find_all(name='li'):
                            li_valid_filed_count = self.get_dom_field_count(li)
                            if li_valid_filed_count < 2:
                                break
                        if len(ul.text.strip().split("\n")) > 1 and li_valid_filed_count > 1:
                            is_has_filter_value = False
                            # li text判断是否有过滤词,如果有过滤词，认为不是需要采集的业务数据
                            for li in ul.find_all(name='li'):
                                is_has_filter_value = self.is_has_filter_value_in_dom_text(li)
                                if is_has_filter_value:
                                    break
                            if is_has_filter_value is False:
                                # 判断每个li下的字段个数差异是否在阈值范围内如15，这样程序才认为是规范化的列表数据
                                if self.check_each_li_field_count_diff_in_range(ul, 15):
                                    print('----------------------------')
                                    # print(str(ul.text).encode("UTF-8").decode().strip().replace('\n', '').replace('\t', ''))
                                    css_select_path_head = ul.parent.name + self.get_attrbuite_str(
                                        ul.parent.attrs) + ">"
                                    css_select_path_head += ul.name + self.get_attrbuite_str(
                                        ul.attrs) + ">"
                                    field_obj_list = self.get_dom_contain_element_css_selector_and_field_old(ul,
                                                                                                             css_select_path_head)

                                    all_recog_data_field_list.append(field_obj_list)
        # 获取每个抓取字段内容并转化成excel需要的数据对象
        all_recog_data_record_list = self.get_excel_value_from_recog_field(soup, all_recog_data_field_list)
        return all_recog_data_record_list

    # 自动解析网页中的table数据
    # 每个td的css selector路径都一样，导致每列无法定位,所以需要认为增加td1,td2之类
    def parse_table_in_html(self):
        # htmlinfo = self.get_web_page_html()
        soup = BeautifulSoup(self.htmlinfo, 'html.parser')
        # print("输出htmlinfo:"+str(self.htmlinfo, encoding = "utf-8"))
        # 所有识别结果的数据记录数组
        all_recog_data_record_list = []
        # 所有识别结果的字段信息数组（字段名，字段路径）
        all_recog_data_field_list = []
        for table in soup.find_all(name='table'):
            print("循环每个table")
            # print(table.contents)
            # print(str(table.contents[0]).encode("UTF-8").decode())
            # table里文本不能为空，或者换行符
            if table.text.strip() != "" and table.text.strip() != "\n":
                # table下的tr数量要超过1个，即有多行
                if len(table.find_all(name='tr')) > 1:
                    print("table 下面 tr超过一行")
                    # # tr的子元素个数大于1，即有多列
                    # # 有可能第一个tr里的td个数（合并单元格后）跟其他tr的td个数不同，所以从第二行开始计算td个数
                    # iterator_operator = IteratorOperate(table.find(name='tr').children)
                    # iterator_count = iterator_operator.get_iterator_count()
                    # if iterator_count > 1:
                    # 第二行开始的tr有效字段个数，即text内容有效值数量超过1个
                    tr_valid_filed_count = 0
                    tr_row = 0
                    tr_list_obj = self.remove_except_tr_in_table(soup, table.find_all(name='tr'))
                    for tr in tr_list_obj:
                        tr_row += 1
                        if tr_row > 1:
                            tr_valid_filed_count = self.get_tr_field_count(soup, tr)
                            if tr_valid_filed_count > 1:
                                break
                    # if len(table.text.strip().split("\n")) > 1 and tr_valid_filed_count > 1:
                    # 部分表格table.text返回值没有用\n分割
                    if tr_valid_filed_count > 1:
                        print("table 有效字段个数超过1")
                        is_has_filter_value = False
                        # tr text判断是否有过滤词,如果有过滤词，认为不是需要采集的业务数据
                        # for tr in table.find_all(name='tr'):
                        #     is_has_filter_value = self.is_has_filter_value_in_dom_text(tr)
                        #     if is_has_filter_value:
                        #         break
                        if is_has_filter_value is False:
                            # 判断第二行开始的tr下的字段个数是否在阈值范围内如0,即每行的列数都相同，这样程序才认为是规范化的表格数据
                            if self.check_each_tr_field_count_diff_in_range(soup, table, 0):
                                print('----------------------------')
                                # print(str(table.text).encode("UTF-8").decode().strip().replace('\n', '').replace('\t', ''))
                                # print(table.text.strip().replace('\n', '').replace('\t', ''))
                                # css_select_path_head = table.parent.name + self.get_attrbuite_str(
                                #     table.parent.attrs) + ">"
                                # css_select_path_head += table.name + self.get_attrbuite_str(
                                #     table.attrs) + ">"
                                css_select_path_head = self.get_dom_parent_css_slector_util_top(table)
                                css_select_path_head += ">"
                                # 得到tr的路径，每个td字段的path一次为 td[0],td[1],针对table数据做特殊处理，在解析时解析后再对每个td字段进行定位
                                field_obj_list = self.get_table_contain_tr_path_and_field(soup, table,
                                                                                          css_select_path_head)
                                all_recog_data_field_list.append(field_obj_list)
        # 获取每个抓取字段内容并转化成excel需要的数据对象
        all_recog_data_record_list = self.get_excel_value_from_recog_field(soup, all_recog_data_field_list)
        return all_recog_data_record_list

    # 利用pandas的方法直接读取table，但与ul和div的数据读取流程不一致
    def pares_table_in_html_by_pandas(self):
        # import pandas as pd
        # url = 'http://www.kuaidaili.com/free/'
        # df = pd.read_html(url)[0]
        pass

    # 自动解析网页中的div数据，相同class
    def parse_div_class_in_html(self):
        # htmlinfo = self.get_web_page_html()
        soup = BeautifulSoup(self.htmlinfo, 'html.parser')
        # tt = soup.select('div.key-list.imglazyload>div.item-mod>div.infos>a.lp-name>span.items-name')
        div_class_obj = {}
        for div in soup.find_all(name='div'):
            if div.attrs.get("class") != None:
                div_class_name = self.convert_dom_class_str_from_array(div.attrs.get("class"))
                # print(div.attrs.get("class"))
                if div_class_name not in div_class_obj:
                    div_class_obj[div_class_name] = 1
                else:
                    div_class_obj[div_class_name] += 1
        # 对出现频次大于5个div class进行解析
        new_div_class_obj = {key: value for key, value in div_class_obj.items() if value > 5}
        print(new_div_class_obj)
        # 所有识别结果的div存储的数组
        all_need_div_content_list = []
        # 所有识别结果的数据记录数组
        all_recog_data_record_list = []
        # 所有识别结果的字段信息数组（字段名，字段路径）
        all_recog_data_field_list = []
        # 循环筛选后的不同class的div
        for key, value in new_div_class_obj.items():
            # 获取相同class的div
            all_class_div_content = soup.find_all("div", attrs={"class": key})
            print("---class--------------------------------------------------------------->" + key)
            # 对这些相同calss div进行分割，分割原则为是否为属于相同parent
            split_div_conent_same_class = self.split_div_list_into_same_parent(all_class_div_content)
            for div_content_same_class in split_div_conent_same_class:
                # 判断相同parent的相同class div，有效字段数的差值是否在阈值范围内，如40
                if self.check_div_field_count_diff_in_range(div_content_same_class, 40):
                    need_div_content_list = []
                    for div_content in div_content_same_class:
                        if div_content.text is not None and div_content.text.strip() != "":
                            # 判断div对象是否有并列的同胞，如果没有则认为不是循环的数据
                            if div_content.find_next_sibling() is not None:
                                # 判断并列的同胞div是否具有相同的class
                                is_has_same_class = self.check_two_dom_has_same_class(div_content,
                                                                                      div_content.find_next_sibling())
                                if is_has_same_class:
                                    # 判断div的有效字段个数是否大于0，如果这些字段路径相同则暂时判断不出有效的字段个数
                                    valid_field_count = self.get_dom_field_count(div_content)
                                    if valid_field_count > 0:
                                        print("|||||||||||||||||||||||||||||||||||||||||||||||||||")
                                        # print(str(div_content.text).encode("UTF-8").decode().strip().replace('\n', '').replace('\t', ''))
                                        # print(div_content.text.strip().replace('\n', '').replace('\t', ''))
                                        need_div_content_list.append(div_content)
                    if len(need_div_content_list) > 0:
                        all_need_div_content_list.append(need_div_content_list)
        # 解析得到的不同识别结果，对每个元素获取css-selector
        # soup.select('div.key-list.imglazyload>div.item-mod>div.infos>a.lp-name>span.items-name')
        # 获取div class下的抓取字段和每个字段访问路径
        for div_class_array in all_need_div_content_list:
            each_recog_data_field_list = []
            # print(div_class_array)
            div_class = div_class_array[0]
            # print(div_class)

            css_select_path_head = div_class.parent.name + self.get_attrbuite_str(div_class.parent.attrs) + ">"
            css_select_path_head += div_class.name + self.get_attrbuite_str(div_class.attrs) + ">"
            # print(css_select_path_head)
            field_obj_list = self.get_dom_contain_element_css_selector_and_field(div_class, css_select_path_head)

            # css_xpath_head = "//{}[@class='{}']".format(div_class.name, self.get_attrbuite_str2(div_class.attrs)) + "/"
            # field_obj_list = self.get_dom_contain_element_xpath_and_field(div_class, css_xpath_head)

            all_recog_data_field_list.append(field_obj_list)
        # 获取每个抓取字段内容并转化成excel需要的数据对象
        all_recog_data_record_list = self.get_excel_value_from_recog_field(soup, all_recog_data_field_list)
        return all_recog_data_record_list

    # 对返回的dom对象class数组进行拼接还原，中间用空格
    def convert_dom_class_str_from_array(self, class_array):
        class_str = ""
        for split_class in class_array:
            class_str += split_class + " "
        class_str = class_str.rstrip(" ")
        return class_str

    # 获取识别字段的字段内容信息并转化为excel生成需要的数据对象
    def get_excel_value_from_recog_field(self, soup, all_recog_data_field_list):
        all_recog_data_record_list = []
        # 针对路径相同的字段进行字段去重
        all_recog_data_field_list = self.remain_one_field_of_same_path(all_recog_data_field_list)
        # 获取div class下的每个抓取字段的内容
        # 循环识别结果
        all_recog_field_column_value_list = []
        for i in range(len(all_recog_data_field_list)):
            # 获取每个识别结果的列（字段）
            each_recog_field_column_value_list = []
            for w in range(len(all_recog_data_field_list[i])):
                single_field_obj = all_recog_data_field_list[i][w]
                # {"字段旧名称":"字段1","字段新名称":"地址","字段路径":"div.key-list.imglazyload>div.item-mod>div.infos>a.lp-name>span.items-name"
                # {"字段旧名称":"字段1","字段新名称":"地址","字段路径":"//div[@class="search-item sv-search-company"]/div/div[3]/div[5]/div/span[2]"
                single_field_path = single_field_obj.get("字段路径")
                # 得到每个字段的所有记录，相当于数据表的一列
                if "ThisIsTable.HeadRow." not in single_field_path:
                    single_field_cloumn_value_list = soup.select(single_field_path)
                    # single_field_cloumn_value_list = etree.HTML(soup.prettify()).xpath(single_field_path)
                # 针对表格数据特别处理
                else:
                    single_field_cloumn_value_list = self.get_table_td_value_list(soup, single_field_path)
                each_recog_field_column_value_list.append(single_field_cloumn_value_list)
            all_recog_field_column_value_list.append(each_recog_field_column_value_list)
        # 赋值形成excel需要的数组对象
        # 循环识别结果
        for j in range(len(all_recog_field_column_value_list)):
            each_recog_data_record_list = []
            each_recog_field_column_value_list = all_recog_field_column_value_list[j]
            # 抓取数据至少有两个字段
            if len(each_recog_field_column_value_list) > 1:
                # 获取到字段中的最多行数
                record_rows = self.get_max_row_records(each_recog_field_column_value_list)
                # 循环每个识别结果的行
                for h in range(record_rows):
                    # 针对第一行，添加字段的路径
                    if h == 0:
                        tmp_obj = {}
                        for u in range(len(each_recog_field_column_value_list)):
                            tmp_single_field_obj = all_recog_data_field_list[j][u]
                            tmp_single_filed_new_name = tmp_single_field_obj.get("字段新名称")
                            tmp_obj[tmp_single_filed_new_name] = tmp_single_field_obj.get("字段路径")
                        each_recog_data_record_list.append(tmp_obj)
                    obj = {}
                    # 循环每个识别结果的列（字段）
                    for k in range(len(each_recog_field_column_value_list)):
                        single_field_obj = all_recog_data_field_list[j][k]
                        single_filed_new_name = single_field_obj.get("字段新名称")
                        single_field_path = single_field_obj.get("字段路径")
                        # 需要保证每个字段的记录行数都是一样的，否则会报错
                        try:
                            cell_obj = each_recog_field_column_value_list[k][h]
                            if "ThisIsTable.HeadRow." not in single_field_path:
                                obj[single_filed_new_name] = self.get_field_value_contain_tails(cell_obj)
                            else:
                                obj[single_filed_new_name] = cell_obj.get("text").lstrip("\n").lstrip(" ").rstrip(
                                    " ").rstrip("\n")
                        except Exception as e:
                            obj[single_filed_new_name] = ""
                            pass
                        continue
                    each_recog_data_record_list.append(obj)
                all_recog_data_record_list.append(each_recog_data_record_list)
        return all_recog_data_record_list

    # 获取抓取的字段值（除了text外，把前后的tail也包含捡来）
    # 如：舞钢广众<font color="red">钢铁</font>有限公司
    def get_field_value_contain_tails(self, cell_obj):
        field_value = ""
        field_value = cell_obj.text.lstrip("\n").lstrip(" ").rstrip(" ").rstrip("\n")
        if type(cell_obj.nextSibling) == NavigableString:
            field_value += cell_obj.nextSibling.lstrip("\n").lstrip(" ").rstrip(" ").rstrip("\n")
        if type(cell_obj.previousSibling) == NavigableString:
            field_value = cell_obj.previousSibling.lstrip("\n").lstrip(" ").rstrip(" ").rstrip("\n") + field_value
        return field_value

    # 获取表格td单列的值
    def get_table_td_value_list(self, soup, td_path):
        single_field_cloumn_value_list = []
        split_array = td_path.split(">")
        tr_path = ""
        for i in range(len(split_array)):
            if i < len(split_array) - 2:
                tmp_path = split_array[i] + ">"
                tr_path += tmp_path
        tr_path = tr_path.rstrip(">")
        td_index = int(split_array[len(split_array) - 2].lstrip("td[").rstrip("]"))
        head_row_num = int(split_array[len(split_array) - 1].lstrip("ThisIsTable.HeadRow."))
        tr_list = soup.select(tr_path)
        # 去除异常的tr，如tr字段数相差10倍，tr字段合并单元格等情况
        tr_list = self.remove_except_tr_in_table(soup, tr_list)
        for t in range(len(tr_list)):
            tr_obj = tr_list[t]
            if t > head_row_num:
                tr_obj_td_list = tr_obj.find_all(name='td')
                for w in range(len(tr_obj_td_list)):
                    if w == td_index:
                        field_column_value_obj = {}
                        field_column_value_obj["text"] = tr_obj_td_list[w].get_text().strip()
                        single_field_cloumn_value_list.append(field_column_value_obj)
        return single_field_cloumn_value_list

    # 获取最多的行数
    def get_max_row_records(self, each_recog_field_column_value_list):
        row_num = len(each_recog_field_column_value_list[0])
        for item in each_recog_field_column_value_list:
            if len(item) > row_num:
                row_num = len(item)
        return row_num

    # 将相同路径的字段全部去掉
    def remove_all_field_of_same_path(self, all_recog_data_field_list):
        for j in range(len(all_recog_data_field_list)):
            field_obj_list = all_recog_data_field_list[j]
            field_obj_path_array = []
            remove_index = []
            for w in range(len(field_obj_list)):
                field_obj_path = field_obj_list[w].get("字段路径")
                field_obj_path_array.append(field_obj_path)
            for k in range(len(field_obj_path_array)):
                field_obj_path = field_obj_path_array[k]
                if field_obj_path_array.count(field_obj_path) > 1:
                    remove_index.append(k)
            cnt = 0
            for index in remove_index:
                del all_recog_data_field_list[j][index - cnt]
                cnt += 1
        return all_recog_data_field_list

    # 将相同路径的字段去掉重复的，只保留一个
    def remain_one_field_of_same_path(self, all_recog_data_field_list):
        for j in range(len(all_recog_data_field_list)):
            field_obj_list = all_recog_data_field_list[j]
            field_obj_path_array = []
            remove_index = []
            for w in range(len(field_obj_list)):
                field_obj_path = field_obj_list[w].get("字段路径")
                if field_obj_path in field_obj_path_array:
                    remove_index.append(w)
                else:
                    field_obj_path_array.append(field_obj_path)
            cnt = 0
            for index in remove_index:
                del all_recog_data_field_list[j][index - cnt]
                cnt += 1
        return all_recog_data_field_list

    # 将多个attr用.连接
    def get_attrbuite_str(self, attrs):
        attr_str = ""
        attr_array = attrs.get("class")
        if attr_array is not None:
            for attr in attr_array:
                attr_str += attr + "."
            attr_str = attr_str.rstrip(".")
        if attr_str != "":
            attr_str = "." + attr_str
        return attr_str

    # 将多个attr用空格连接
    def get_attrbuite_str2(self, attrs):
        attr_str = ""
        attr_array = attrs.get("class")
        if attr_array is not None:
            for attr in attr_array:
                attr_str += attr + " "
            attr_str = attr_str.rstrip(" ")
        # if attr_str != "":
        #     attr_str = "." + attr_str
        return attr_str

    # 针对dom对象（如<table>)逐级往上获取父元素，直到顶部元素<html>
    def get_dom_parent_css_slector_util_top(self, current_dom_obj):
        is_not_top = True
        css_slector = ""
        while is_not_top:
            if current_dom_obj.name == "html":
                is_not_top = False
            else:
                css_slector = current_dom_obj.name + self.get_attrbuite_str(
                    current_dom_obj.attrs) + ">" + css_slector
                current_dom_obj = current_dom_obj.parent
        css_slector = css_slector.rstrip(">")
        return css_slector

    # 得到tr下的td元素的字段路径css-selector
    def get_table_contain_tr_path_and_field(self, soup, table_obj, css_selector_head):
        field_obj_list = []
        # tr_list = table_obj.find_all(name='tr')
        tr_list = self.remove_except_tr_in_table(soup, table_obj.find_all(name='tr'))
        is_has_th = False
        head_row_num = 0
        # 表头在tr/th
        if len(tr_list[0].find_all(name='th')) > 0:
            is_has_th = True
            head_row_num = 0
        # 表头在tr/td，如果head_row_num=-1则没有表头
        else:
            head_row_num = self.get_head_tr_of_table(soup, table_obj)
        head_tr = tr_list[head_row_num]
        body_tr = tr_list[head_row_num + 1]
        head_tr_td_list = None
        if is_has_th:
            head_tr_td_list = head_tr.find_all(name='th')
        else:
            head_tr_td_list = head_tr.find_all(name='td')
        # body_tr_td_list = body_tr.find_all(name='td')
        for i in range(len(head_tr_td_list)):
            field_obj = {}
            if head_row_num != -1:
                if head_tr_td_list[i].get_text().strip() != "":
                    field_obj["字段旧名称"] = head_tr_td_list[i].get_text().strip()
                else:
                    field_obj["字段旧名称"] = "字段" + str(i + 1)
                field_obj["字段新名称"] = field_obj["字段旧名称"]
            else:
                field_obj["字段旧名称"] = "字段" + str(i + 1)
                field_obj["字段新名称"] = "字段" + str(i + 1)
            field_obj["字段路径"] = css_selector_head + self.get_parent_css_slector_util_top_old(table_obj,
                                                                                             body_tr) + ">td[" + str(
                i) + "]>ThisIsTable.HeadRow." + str(head_row_num)
            field_obj_list.append(field_obj)
        return field_obj_list

    # 得到表格的头行，即包含字段名称的那行
    # 针对合并单元格的表格，表头不一定在第一行
    # 如果没有表头行返回-1    #
    def get_head_tr_of_table(self, soup, table_obj):
        # tr_list = table_obj.find_all(name='tr')
        tr_list = self.remove_except_tr_in_table(soup, table_obj.find_all(name='tr'))
        td_count = len(tr_list[len(tr_list) - 1].find_all(name='td'))
        head_row_num = -1
        for i in range(len(tr_list)):
            if len(tr_list[i].find_all(name='td')) == td_count:
                head_row_num = i
                # 如果表头内容过长，那有可能就是没有表头
                tr_td_list = tr_list[i].find_all(name='td')
                for w in range(len(tr_td_list)):
                    if len(tr_td_list[w].get_text().strip()) > 10:
                        head_row_num = -1
                        break
                break
        return head_row_num

    # 得到每个元素的字段路径css-selector，给ul用
    def get_dom_contain_element_css_selector_and_field_old(self, dom_obj, css_selector_head):
        field_obj_list = []
        field_index = 1
        # 获取当前元素的所有子孙节点
        descendants = dom_obj.descendants
        for descendant in descendants:
            if type(descendant) != NavigableString and type(descendant) != Comment and len(descendant.contents) == 1:
                if descendant.text.replace('\n', '').strip() != "":
                    # 找到了有效的字段内容
                    field_obj = {}
                    field_obj["字段旧名称"] = "字段" + str(field_index)
                    field_obj["字段新名称"] = "字段" + str(field_index)
                    field_obj["字段路径"] = css_selector_head + self.get_parent_css_slector_util_top_old(dom_obj,
                                                                                                     descendant)
                    field_obj_list.append(field_obj)
                    field_index += 1
        return field_obj_list

    # 得到每个元素的字段路径css-selector，给div用
    def get_dom_contain_element_css_selector_and_field(self, dom_obj, css_selector_head):
        field_obj_list = []
        field_index = 1
        # 获取当前元素的所有子孙节点
        descendants = dom_obj.descendants
        for descendant in descendants:
            if type(descendant) != NavigableString and type(descendant) != Comment and hasattr(descendant,
                                                                                               "contents") and len(
                descendant.contents) == 1:
                if descendant.text.replace('\n', '').strip() != "":
                    # 找到了有效的字段内容
                    field_obj = {}
                    field_obj["字段旧名称"] = "字段" + str(field_index)
                    field_obj["字段新名称"] = "字段" + str(field_index)
                    field_obj["字段路径"] = css_selector_head + self.get_parent_css_slector_util_top(dom_obj, descendant)
                    field_obj_list.append(field_obj)
                    field_index += 1
        return field_obj_list

    # 得到每个元素的字段路径xpath,已作废
    def get_dom_contain_element_xpath_and_field(self, dom_obj, css_xpath_head):
        field_obj_list = []
        field_index = 1
        # 获取当前元素的所有子孙节点
        descendants = dom_obj.descendants
        for descendant in descendants:
            if type(descendant) != NavigableString and type(descendant) != Comment and len(descendant.contents) == 1:
                if descendant.text.replace('\n', '').strip() != "":
                    # 找到了有效的字段内容
                    field_obj = {}
                    field_obj["字段旧名称"] = "字段" + str(field_index)
                    field_obj["字段新名称"] = "字段" + str(field_index)
                    field_obj["字段路径"] = css_xpath_head + self.get_parent_css_xpath_util_top(dom_obj, descendant)
                    field_obj_list.append(field_obj)
                    field_index += 1
        return field_obj_list

    # 逐级往上获取各个父元素的css selector，直接到顶层元素,----给table,ul用
    def get_parent_css_slector_util_top_old(self, top_dom_obj, current_dom_obj):
        is_not_top = True
        css_slector = ""
        while is_not_top:
            if current_dom_obj.name == top_dom_obj.name and current_dom_obj.attrs == top_dom_obj.attrs:
                is_not_top = False
            else:
                css_slector = current_dom_obj.name + self.get_attrbuite_str(
                    current_dom_obj.attrs) + ">" + css_slector
                # css_slector = self.find_index_of_dom_in_same_level2(current_dom_obj) + ">" + css_slector
                current_dom_obj = current_dom_obj.parent
        css_slector = css_slector.rstrip(">")
        return css_slector

    # 逐级往上获取各个父元素的css selector，直接到顶层元素,给div用
    def get_parent_css_slector_util_top(self, top_dom_obj, current_dom_obj):
        is_not_top = True
        css_slector = ""
        while is_not_top:
            if current_dom_obj.name == top_dom_obj.name and current_dom_obj.attrs == top_dom_obj.attrs:
                is_not_top = False
            else:
                # css_slector = current_dom_obj.name + self.get_attrbuite_str(
                #     current_dom_obj.attrs) + ">" + css_slector
                css_slector = self.find_index_of_dom_in_same_level2(current_dom_obj) + ">" + css_slector
                current_dom_obj = current_dom_obj.parent
        css_slector = css_slector.rstrip(">")
        return css_slector

    # 逐级往上获取各个父元素的css xpath，直接到顶层元素，已作废
    def get_parent_css_xpath_util_top(self, top_dom_obj, current_dom_obj):
        is_not_top = True
        css_xpath = ""
        while is_not_top:
            if current_dom_obj.name == top_dom_obj.name and current_dom_obj.attrs == top_dom_obj.attrs:
                is_not_top = False
            else:
                css_xpath = self.find_index_of_dom_in_same_level(current_dom_obj) + "/" + css_xpath
                current_dom_obj = current_dom_obj.parent
        css_xpath = css_xpath.rstrip("/")
        return css_xpath

    # 获取dom对象在同一级出现的索引，从1开始计算，即计算出是第几个兄弟节点标签,适用于xpath，已作废
    def find_index_of_dom_in_same_level(self, dom_obj):
        dom_tag = ""
        if dom_obj.previous_siblings is not None and self.get_count_siblings(dom_obj.previous_siblings) > 0:
            index = self.get_count_siblings(dom_obj.previous_siblings) + 1
            dom_tag = dom_obj.name + "[" + str(index) + "]"
        else:
            if (dom_obj.next_siblings is not None and self.get_count_siblings(dom_obj.next_siblings) > 0):
                dom_tag = dom_obj.name + "[1]"
            else:
                dom_tag = dom_obj.name
        return dom_tag

    # 获取dom对象在同一级出现的索引，从1开始计算，即计算出是第几个兄弟节点标签，适用于css selector，给div用
    def find_index_of_dom_in_same_level2(self, dom_obj):
        dom_tag = ""
        if dom_obj.previous_siblings is not None and self.get_count_siblings(dom_obj.previous_siblings) > 0:
            index = self.get_count_siblings(dom_obj.previous_siblings) + 1
            dom_tag = dom_obj.name + ":nth-child(" + str(index) + ")"
        else:
            if (dom_obj.next_siblings is not None and self.get_count_siblings(dom_obj.next_siblings) > 0):
                dom_tag = dom_obj.name + ":nth-child(1)"
            else:
                dom_tag = dom_obj.name
        return dom_tag

    # 获取有效siblings数组对象的个数
    def get_count_siblings(self, siblings_list):
        # 没有tag name的标签，不计算索引值，如<span>微信</span>或<span>扫一扫</span>，其中 或 不做考虑
        count = 0
        for next in siblings_list:
            if next.name is not None and next.name.strip() != "":
                count += 1
        return count

    # 基于确定的识别结果集规则进行数据采集
    def spider_data_by_confirm_rule(self, field_name_str, field_path_str):
        # self.get_web_page_html()
        soup = BeautifulSoup(self.htmlinfo, 'html.parser')
        # 根据识别结果序号，需要采集的字段及名称，进行html标签定位，并做翻页(，如下一页，>）
        field_name_list = field_name_str.split(",")
        field_path_list = field_path_str.split("&&")
        all_recog_data_field_list = []
        field_obj_list = []
        for i in range(len(field_name_list)):
            field_name = field_name_list[i]
            field_path = field_path_list[i]
            field_obj = {}
            field_obj["字段新名称"] = field_name
            field_obj["字段路径"] = field_path
            field_obj_list.append(field_obj)
        all_recog_data_field_list.append(field_obj_list)
        # 获取每个抓取字段内容并转化成excel需要的数据对象
        all_recog_data_record_list = self.get_excel_value_from_recog_field(soup, all_recog_data_field_list)
        return all_recog_data_record_list

    # 自动翻页，并获取数据，在chrome浏览器里不断点击下一页，直接没有为止,或者根据传入页数来做控制翻到多少页
    def auto_turn_next_page_and_get_data_in_page(self, all_crawler_data_record_list, website_url, fields_name_str,
                                                 fields_path_str, turn_page_num):
        # 针对静态数据网站，在获取第一页数据时没有用web_driver
        # 针对动态数据网站，在获取第一页数据时使用了web_driver
        if self.web_driver == None:
            chrome_options = Options()  # 实例化Option对象
            chrome_options.add_argument('--headless')  # 把Chrome浏览器设置为静默模式
            chrome_options.add_argument('--disable-gpu')  # 禁止加载图片
            driver = webdriver.Chrome(options=chrome_options)  # 设置引擎为Chrome，在后台默默运行
            driver.get(website_url)
            self.web_driver = driver
        # 下一页url，静态数据网站返回不同url，动态数据网站返回的url不变，因此就模拟点击下一页操作一直在浏览器中执行即可
        next_page_element = self.get_next_page_togger(self.web_driver)
        if next_page_element is not None:
            is_not_last_page = True
            curent_page_num = 1
            while is_not_last_page and curent_page_num <= turn_page_num:
                try:
                    next_page_element = self.get_next_page_togger(self.web_driver)
                    self.web_driver.execute_script("arguments[0].click();", next_page_element)
                    curent_page_num += 1
                    print("自动获取到第{}页-----".format(str(curent_page_num)))
                    if self.is_daynamic_data:
                        self.set_curent_page_html(self.web_driver.page_source)
                    # 如果是静态数据网站，页面html得用request得到，这样才能和第一次识别提取到的路径对应上
                    else:
                        next_page_url = self.web_driver.current_url
                        r = requests.get(url=next_page_url, headers=headers)
                        r.encoding = r.apparent_encoding
                        # r.encoding = 'utf-8'
                        self.set_curent_page_html(r.content.decode())
                        self.set_curent_page_html(r.text)
                    temp_data_record_list = self.spider_data_by_confirm_rule(fields_name_str, fields_path_str)
                    # 删除表格的第一行（字段路径）
                    if len(temp_data_record_list) > 0 and len(temp_data_record_list[0]) > 0:
                        del temp_data_record_list[0][0]
                        all_crawler_data_record_list += temp_data_record_list
                    else:
                        print("当前页没有抓取到数据")
                except Exception as exp:
                    # 通过没有下一页的提示错误来分析，是否到了最后一页
                    if "Cannot read property 'click' of null" in str(exp):
                        print("当前已到最后一页")
                    else:
                        print("翻页错误：{}".format(exp))
                    is_not_last_page = False

                    pass
        else:
            print("没有找到翻页按钮")
        # driver.quit()
        # driver.close()
        return all_crawler_data_record_list

    # 获取翻页的下一页的按钮,如果没有为None
    def get_next_page_togger(self, web_driver):
        next_page_dom_tag = ["a", "span"]
        next_page_text = ["下一页", "下页"]
        next_page_element = None
        is_not_find_next = True
        for tag in next_page_dom_tag:
            for txt in next_page_text:
                next_page_path = "//{}[contains(text(),'{}')]".format(tag, txt)
                try:
                    next_page_element = web_driver.find_element_by_xpath(next_page_path)
                    is_not_find_next = False
                    break
                except Exception as e:
                    pass
                continue
            if is_not_find_next == False:
                break
        return next_page_element

    # 将所有数组合并成一个数组
    def convert_all_list_into_one_list(self, all_data_record_list):
        result_data_list = []
        for each_data_record_list in all_data_record_list:
            for data_field_value_obj in each_data_record_list:
                result_data_list.append(data_field_value_obj)
        return result_data_list

    # 获取元素如li,div下面有效字段的个数
    def get_dom_field_count(self, dom_obj):
        filed_count = 0
        dom_text_value_list = dom_obj.text.split("\n")
        for each_text in dom_text_value_list:
            if each_text.replace('\n', '').strip() != "":
                filed_count += 1
        return filed_count

    # 获取tr下面的th或td个数
    def get_tr_field_count(self, soup, tr_obj):
        td_list = tr_obj.find_all(name="td")
        if len(td_list) == 0:
            td_list = tr_obj.find_all(name="th")
        return len(td_list)

    # 判断元素如li,div里是否有过滤词
    def is_has_filter_value_in_dom_text(self, dom_obj):
        is_has = False
        dom_text_value_list = dom_obj.text.split("\n")
        if list(set(dom_text_value_list) & set(filterValueArray)):
            is_has = True
        return is_has

    # 判断ul下面的每个li字段数量差异，如果超过阈值，则认为这些ul数值不是要采集的数据
    def check_each_li_field_count_diff_in_range(self, ul_obj, thre_num):
        is_diff_in_range = True
        obj = {}
        li_list_obj = ul_obj.find_all(name='li')
        for i in range(len(li_list_obj)):
            field_count = self.get_dom_field_count(li_list_obj[i])
            obj[i] = field_count
        if obj is not None:
            max_value = max(obj.values())
            min_value = min(obj.values())
            if max_value - min_value > thre_num:
                is_diff_in_range = False
        return is_diff_in_range

    # 判断table下面的每个tr字段数量差异，如果超过阈值，则认为这些table数值不是要采集的数据
    def check_each_tr_field_count_diff_in_range(self, soup, table_obj, thre_num):
        is_diff_in_range = True
        obj = {}
        # tr_list_obj = table_obj.find_all(name='tr')
        tr_list_obj = self.remove_except_tr_in_table(soup, tr_list_obj=table_obj.find_all(name='tr'))
        for i in range(len(tr_list_obj)):
            if i > 0:
                field_count = self.get_tr_field_count(soup, tr_list_obj[i])
                obj[i] = field_count
        if obj is not None:
            max_value = max(obj.values())
            min_value = min(obj.values())
            if max_value - min_value > thre_num:
                is_diff_in_range = False
        return is_diff_in_range

    # tr的字段个数如果相差10倍，则认为这个tr可能是嵌套（tr外面的tr）, 则认为是异常tr,不参与计算
    # tr的字段个数，如果和别的不同，则是合并单元格，这种tr去掉
    def remove_except_tr_in_table(self, soup, tr_list_obj):
        # tr_list_obj = table_obj.find_all(name='tr')
        # 排查表格末尾是合并单元格的情况
        last_record_field_count = self.get_tr_field_count(soup, tr_list_obj[len(tr_list_obj) - 2])
        # 去除 相差10倍以上的tr，嵌套tr
        remove_index = []
        for i in range(len(tr_list_obj)):
            # if i > 0:
            field_count = self.get_tr_field_count(soup, tr_list_obj[i])
            # tr的字段个数如果相差10倍，则认为这个tr可能是嵌套（tr外面的tr）,不参与计算
            if field_count / last_record_field_count > 10:
                remove_index.append(i)
            # tr字段做了单元格合并
            if field_count < last_record_field_count:
                remove_index.append(i)
        cnt = 0
        for index in remove_index:
            del tr_list_obj[index - cnt]
            cnt += 1
        # 去除 合并单元格
        return tr_list_obj

    # 判断ul下的每个li字段数值是否相同
    def check_eqaul_each_li_field_count(self, ul_obj):
        is_equal = True
        field_count = self.get_dom_field_count(ul_obj.find(name='li'))
        for li in ul_obj.find_all(name='li'):
            pre_field_count = field_count
            field_count = self.get_dom_field_count(li)
            if field_count != pre_field_count:
                is_equal = False
                break
        return is_equal

    # 对同class的所有div下的有效字段个数进行判断，如果超过阈值6，则认为这些div数值不是要采集的数据
    def check_div_field_count_diff_in_range(self, div_obj_list, thre_num):
        is_diff_in_range = True
        obj = {}
        for i in range(len(div_obj_list)):
            field_count = self.get_dom_field_count(div_obj_list[i])
            obj[i] = field_count
        if obj is not None:
            max_value = max(obj.values())
            min_value = min(obj.values())
            if max_value - min_value > thre_num:
                is_diff_in_range = False
        return is_diff_in_range

    # 判断两个元素是否有相同class
    def check_two_dom_has_same_class(self, current_dom_obj, next_dom_obj):
        current_dom_class_name = ""
        next_dom_class_name = ""
        if current_dom_obj.attrs.get("class") != None:
            current_dom_class_name = str(current_dom_obj.attrs.get("class")).lstrip("['").rstrip("']")
        if next_dom_obj.attrs.get("class") != None:
            next_dom_class_name = str(next_dom_obj.attrs.get("class")).lstrip("['").rstrip("']")
        if current_dom_class_name == next_dom_class_name:
            return True
        else:
            return False

    # 将相同class的所有div再进行分组，分组依据为是否为相同parent,且分组后的div个数大于5个
    def split_div_list_into_same_parent(self, div_obj_list):
        div_obj_list_split_list = []
        obj = {}
        obj[div_obj_list[0].parent] = str(0)
        for i in range(1, len(div_obj_list)):
            div_obj = div_obj_list[i]
            if div_obj.parent not in obj:
                obj[div_obj.parent] = str(i)
            else:
                obj[div_obj.parent] = obj[div_obj.parent] + ";" + str(i)
        for key, value in obj.items():
            split_div_obj_list = []
            for index in value.split(";"):
                split_div_obj_list.append(div_obj_list[int(index)])
            if len(split_div_obj_list) > 5:
                div_obj_list_split_list.append(split_div_obj_list)
        return div_obj_list_split_list



#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    : 2021/1/18 10:31
# @Author  : chenxw
# @Email   : gisfanmache.@gmail.com
# @File    :
# @Descr   :
# @Software: PyCharm

class Setting:
    AUTH_STR = "67697366616e5f61694073696e612e636f6d262626726f6f74313233282a265e25"
    AUTH_SPLIT_STR = "&&&"

    # 初始化
    def __init__(self):
        pass

    def get_AUTH_STR(self):
        return self.AUTH_STR

    def set_AUTH_STR(self, auth_str):
        self.AUTH_STR = auth_str

    def get_AUTH_SPLIT_STR(self):
        return self.AUTH_SPLIT_STR

    def set_AUTH_SPLIT_STR(self, split_str):
        self.AUTH_SPLIT_STR = split_str
